@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997 - 2002 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the 
  *    documentation and/or other materials provided with the distribution. 
  *
- * 3. All advertising materials mentioning features or use of this software 
- *    must display the following acknowledgement: 
- *      This product includes software developed by Kungliga Tekniska 
- *      Hgskolan and its contributors. 
- *
- * 4. Neither the name of the Institute nor the names of its contributors 
+ * 3. Neither the name of the Institute nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software 
  *    without specific prior written permission. 
  *
@@ -36,13 +31,21 @@
  * SUCH DAMAGE. 
  */
 
-/* $Id: getarg.h,v 1.7 1999/09/24 14:47:34 joda Exp $ */
+/* $Id: getarg.h,v 1.14 2005/04/13 05:52:27 lha Exp $ */
 
 #ifndef __GETARG_H__
 #define __GETARG_H__
 
 #include <stddef.h>
 
+#ifndef ROKEN_LIB_FUNCTION
+#ifdef _WIN32
+#define ROKEN_LIB_FUNCTION _stdcall
+#else
+#define ROKEN_LIB_FUNCTION
+#endif
+#endif
+
 struct getargs{
     const char *long_name;
     char short_name;
@@ -52,7 +55,8 @@ struct getargs{
 	   arg_negative_flag, 
 	   arg_strings,
 	   arg_double,
-	   arg_collect
+	   arg_collect,
+	   arg_counter
     } type;
     void *value;
     const char *help;
@@ -70,17 +74,29 @@ typedef struct getarg_strings {
     char **strings;
 } getarg_strings;
 
+typedef int (*getarg_collect_func)(int short_opt,
+				   int argc,
+				   char **argv,
+				   int *goptind,
+				   int *goptarg,
+				   void *data);
+
 typedef struct getarg_collect_info {
-    int (*func)(char *optarg, int argc, char **argv, int *optind, void *data);
+    getarg_collect_func func;
     void *data;
 } getarg_collect_info;
 
-int getarg(struct getargs *args, size_t num_args, 
-	   int argc, char **argv, int *optind);
+int ROKEN_LIB_FUNCTION
+getarg(struct getargs *args, size_t num_args, 
+       int argc, char **argv, int *goptind);
 
-void arg_printusage (struct getargs *args,
-		     size_t num_args,
-		     const char *progname,
-		     const char *extra_string);
+void ROKEN_LIB_FUNCTION
+arg_printusage (struct getargs *args,
+		size_t num_args,
+		const char *progname,
+		const char *extra_string);
+
+void ROKEN_LIB_FUNCTION
+free_getarg_strings (getarg_strings *);
 
 #endif /* __GETARG_H__ */
